use std::sync::Arc;

use serde::{Deserialize, Serialize};

use crate::actions::delete::{DeleteContentBackward, DeleteContentForward};
use crate::actions::insert::InsertText;
use crate::actions::linebreak::InsertLineBreak;
use crate::actions::paste::InsertFromPaste;
use crate::actions::ActionRunner;
use crate::AppState;

#[derive(Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
enum ActionType {
    InsertText,
    InsertFromPaste,
    InsertLineBreak,
    DeleteContentBackward,
    DeleteContentForward,
}

#[derive(Serialize, Deserialize)]
pub(crate) struct Action {
    pub(crate) client: String,
    time: u64,
    data: Option<String>,
    action: ActionType,
    start: usize,
    end: usize,
}

impl Action {
    pub(crate) async fn execute(&self, state: Arc<AppState>) {
        {
            let mut doc = state.doc.write().await;
            let action: Box<dyn ActionRunner> = match self.action {
                ActionType::InsertText => Box::new(InsertText),
                ActionType::InsertFromPaste => Box::new(InsertFromPaste),
                ActionType::InsertLineBreak => Box::new(InsertLineBreak),
                ActionType::DeleteContentBackward => Box::new(DeleteContentBackward),
                ActionType::DeleteContentForward => Box::new(DeleteContentForward),
            };

            *doc = action.run(self.start, self.end, self.data.clone(), doc.clone());
        }
    }
}
